#ifndef DLLINTERFACE_H
#define DLLINTERFACE_H

//////////////////////////////////////////////////////////////
//
// Welcome.  Are you sure you want to read this?  We had in
// mind that you wouldn't need to.  Start from the sample
// derived class implementations (ComponentExample.h/cpp) instead.
//
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// Version of the package base class code
//////////////////////////////////////////////////////////////////////////

static const char* PACKAGE_BASE_VERSION = "1.0";

///////////////////////////////////////////////////////////////
// Some plumbing required to integrate your component definitions.
// Out of paranoia about what compilers might do, we:
//	1. actually export only a single DLL function, GetPackageInformation()
//	2. call only C routines (i.e., not object methods) across the DLL interface
//  3. and so there are a slew of function types to be defined so that SMOK and
//			this package can exchange function pointers (which can be thought
//			of as the routine they import/export).
///////////////////////////////////////////////////////////////

class CUserDefComponentBase;

enum HALT_STATUS { SMOK_CONTINUE, SMOK_HALT };

// These structs provide a convenient, table-driven way of describing your
// components' input and output ports.  See the examples of their use in ComponentExample.cpp.
enum INPORTTYPE { NO_INPORT, STD_INPORT, MEM_INPORT };
typedef struct {
	INPORTTYPE	type;
	const char*	name;
	int			width;
    int         isClockdownInput;
} INPORTDESC;

enum OUTPORTTYPE { NO_OUTPORT, COMBINATIONAL_OUTPORT, LATCHED_OUTPORT };
typedef struct {
	OUTPORTTYPE	type;
	const char*	name;
	int			width;
} OUTPORTDESC;

// these are types of routines "imported" by this package -- access to
// smok-side funcationality.
typedef int  (*READMEMPORT_PROC) ( void* pSMOKObj, int portNum, int address, int numBytes );
typedef void (*WRITEMEMPORT_PROC)( void* pSMOKObj, int portNum, int address, int data, int numBytes );

// these are types of routines used to load an individual component type
typedef CUserDefComponentBase* (*CREATE_DEFINED_COMPONENT_PROC_TYPE)();  
typedef bool                   (*GET_DEFINED_COMPONENT_TYPE_PROC)( int type, const char** componentTypeName, 
												                   CREATE_DEFINED_COMPONENT_PROC_TYPE* pCreateProc );

// these are types of routines used to set up an individual component.
// the first obtains information about the component's ports. 
// the second registers callbacks the component can use to maninpulate the smok object.
typedef void (*GETPORTDESCRIPTION_PROC)( CUserDefComponentBase* pObj, INPORTDESC** pInputPorts, OUTPORTDESC** pOutputPorts );
typedef void (*REGISTERCALLBACKS_PROC) ( CUserDefComponentBase* pObj, void* pSMOKObj, READMEMPORT_PROC pReadMemProc, WRITEMEMPORT_PROC pWriteMemProc );

// these are types of package-exported routines that allow the SMOK side to control the 
// package component, mostly on a per-cycle basis
typedef HALT_STATUS	(*COMPUTEOUTPUT_PROC)(CUserDefComponentBase* pObj, int* pInputArray, int* pOutputArray);
typedef HALT_STATUS	(*LATCHINPUTS_PROC)  (CUserDefComponentBase* pObj, int* pInputArray);
typedef bool        (*REINITIALIZE_PROC) (CUserDefComponentBase* pObj);
typedef void        (*DELETEOBJ_PROC)    (CUserDefComponentBase* pObj);

// this is the main package load routine, which obtains descriptive information and 
// pointers to interface functions exported by the package
typedef bool (*PACKAGE_INFO_PROC)(  const char** pBaseVersion,  const char** pVersion,
									const char** pDate, const char** pAuthor, int* numTypes,
									GET_DEFINED_COMPONENT_TYPE_PROC* pGetTypeProc,
									REGISTERCALLBACKS_PROC* pRegisterCallbacksProc,
									GETPORTDESCRIPTION_PROC* pGetPortDescriprtionProc,
									COMPUTEOUTPUT_PROC* pComputeOutputProc,
									LATCHINPUTS_PROC* pLatchInputsProc,
									REINITIALIZE_PROC* pReinitializeProc,
									DELETEOBJ_PROC* pDeleteProc);

///////////////////////////////////////////////////////////////
// Now the exported procedure.
// We restrict ourselves to calling a single C procedure across the
// DLL interface, in a probably futile attempt to minimize
// the difficulty of using a non-MS C++ compiler.
///////////////////////////////////////////////////////////////


extern "C" bool __declspec(dllexport) GetPackageInformation( const char** pBaseVersion, const char** pVersion, 
															 const char** pDate, const char** pAuthor, 
															 int* numTypes, 
															 GET_DEFINED_COMPONENT_TYPE_PROC* pGetTypeProc,
															 REGISTERCALLBACKS_PROC* pRegisterCallbacksProc,
															 GETPORTDESCRIPTION_PROC* pGetPortDescriptionProc,
															 COMPUTEOUTPUT_PROC* pComputeOutputProc,
															 LATCHINPUTS_PROC* pLatchInputsProc,
															 REINITIALIZE_PROC* pReinitializeProc,
															 DELETEOBJ_PROC* pDeleteProc);

#endif // DLLINTERFACE_H